from telegram import Update, ChatPermissions
from telegram.ext import CallbackContext
from .chat import Chat
from typing import Optional, Union
from datetime import datetime


class Moderation(Chat):
    """Class with methods useful for moderating.

    Usage
    -----
        mod = Moderation(update, context)

        # Simple way to get user ID from message you replied with command.
        user_id = update.message["reply_to_message"].from_user.id
        ...
    """

    def __init__(self, update: Update, context: CallbackContext) -> None:
        self.update = update
        self.context = context
        super().__init__(update, context)

    def change_permissions(
        self,
        user_id: int,
        until_date: Union[int, datetime] = None,
        permissions: ChatPermissions = None,
    ) -> None:
        """Change any permission to user in chat."""

        if not self.is_user_admin(user_id):
            self.context.bot.restrict_chat_member(
                self.chat_id, user_id, permissions=permissions, until_date=until_date,
            )
        else:
            raise PermissionError("Cannot change chat admin permissions")

    def mute(self, user_id: int, until_date: Union[int, datetime] = None) -> None:
        """Mute user in chat."""

        perms = ChatPermissions(can_send_messages=False)
        self.change_permissions(user_id, until_date, perms)

    def unmute(self, user_id: int) -> None:
        """Restore permissions to restricted (or muted) user."""

        perms = ChatPermissions(
            can_send_messages=False,
            can_send_media_messages=True,
            can_send_polls=True,
            can_send_other_messages=True,
            can_add_web_page_previews=True,
        )
        self.change_permissions(user_id, permissions=perms)

    def ban(
        self,
        user_id: int,
        until_date: Union[int, datetime] = None,
        revoke_messages: bool = False,
    ) -> None:
        """Ban user in chat."""

        if not self.is_user_admin(user_id):
            self.context.bot.ban_chat_member(
                self.chat_id,
                user_id,
                until_date=until_date,
                revoke_messages=revoke_messages,
            )
        else:
            raise PermissionError("Cannot ban chat admin")

    def unban(self, user_id: int) -> None:
        """Unban banned user in chat."""

        if not self.is_user_admin(user_id):
            self.context.bot.unban_chat_member(self.chat_id, user_id)
        else:
            raise PermissionError("Cannot unban chat admin")



class HelpParser:
    """Parser of help message often sended by /help or /start command."""

    def __init__(self) -> None:
        self.__message = "*Available commands*\n\n"
        self.commands_descriptions = dict()

    def add_command(self, name: str, help: str) -> None:
        self.commands_descriptions[name] = help

    @property
    def content(self) -> str:
        final_message = self.__message
        for cmd, desc in self.commands_descriptions.items():
            final_message += f"/{cmd}\n{desc}\n\n"
        return final_message



import argparse
from . import ProjectInitializer

parser = argparse.ArgumentParser(
    prog="telegrask", description="CLI for Telegrask framework.", allow_abbrev=False,
)
subparsers = parser.add_subparsers(dest="command", required=True)

parser_init = subparsers.add_parser("init", help="initialize project")
parser_init.add_argument(
    "name", metavar="NAME", type=str, nargs=1, help="name for package"
)
parser_init.add_argument(
    "--no-venv",
    dest="venv",
    action="store_false",
    help="do not create virtual environment",
)
parser_init.add_argument(
    "--no-gitignore",
    dest="gitignore",
    action="store_false",
    help="do not include .gitignore file",
)

args = parser.parse_args()
cmd = args.command


def main():
    if cmd == "init":
        ProjectInitializer(args.name[0], args.venv, args.gitignore)



from telegram import Update
from telegram.ext import CallbackContext


class Chat:
    """Class to get basic info about chat in unified object.

    Usage
    -----
        chat = Chat(update, context)
    """

    def __init__(self, update: Update, context: CallbackContext) -> None:
        self.update = update
        self.context = context

        self.chat_id = self.update.message.chat.id

    @property
    def admins(self) -> list:
        admins = self.context.bot.get_chat_administrators(self.chat_id)
        return admins

    @property
    def admins_ids(self) -> list:
        return [admin.user.id for admin in self.admins]

    @property
    def admins_usernames(self) -> list:
        return [admin.user.username for admin in self.admins]

    def is_user_admin(self, user_id: int) -> bool:
        return user_id in self.admins_ids



class Config(dict):
    """Dictionary but without possibility to add new keys."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__dict__ = self

    def __setattr__(self, key, value):
        if key not in [*self.keys(), "__dict__"]:
            raise KeyError("No new keys allowed")
        else:
            super().__setattr__(key, value)

    def __setitem__(self, key, value):
        if key not in self:
            raise KeyError("No new keys allowed")
        else:
            super().__setitem__(key, value)



from .context import ctx
import venv
from os import mkdir, path


class ProjectInitializer:
    def __init__(
        self,
        project_name: str,
        create_venv: bool = True,
        include_gitignore: bool = True,
    ) -> None:
        if create_venv:
            self.create_venv()
        if include_gitignore:
            self.add_gitignore()
        self.create_package(project_name)

    @staticmethod
    def create_venv() -> None:
        venv.create(
            path.join(ctx.app_root, ctx.venv_dir, "venv"),
            system_site_packages=True,
            with_pip=True,
        )

    @staticmethod
    def read_template(template_name: str) -> str:
        with open(path.join(ctx.templates_dir, template_name), "r") as f:
            return f.read()

    @staticmethod
    def write_file(file_path: str, content: str) -> None:
        with open(file_path, "w+") as f:
            f.write(content)

    def add_gitignore(self) -> None:
        template = self.read_template(".gitignore")
        self.write_file(
            path.join(ctx.app_root, ".gitignore"),
            template.replace("__VENV_DIR", ctx.venv_dir),
        )

    def create_package(self, package_name: str) -> None:
        mkdir(path.join(ctx.app_root, package_name))
        package_dir = path.join(ctx.app_root, package_name)
        for template in ["__init__.py", "config.py", "commands.py"]:
            self.write_file(
                path.join(package_dir, template), self.read_template(template)
            )
